"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyToNodeToUpdate = applyToNodeToUpdate;
exports.applyToNodeToDelete = applyToNodeToDelete;
exports.applyToNodeToAdd = applyToNodeToAdd;

var _wasmGen = require("@webassemblyjs/wasm-gen");

var _ast = require("@webassemblyjs/ast");

var _helperWasmSection = require("@webassemblyjs/helper-wasm-section");

var _helperBuffer = require("@webassemblyjs/helper-buffer");

function assertNodeHasLoc(n) {
  if (n.loc == null || n.loc.start == null || n.loc.end == null) {
    throw new Error("Internal failure: can not replace node (".concat(JSON.stringify(n.type), ") without loc information"));
  }
}

function applyToNodeToUpdate(ast, uint8Buffer, nodes) {
  nodes.forEach(function (node) {
    assertNodeHasLoc(node);
    var sectionName = (0, _helperWasmSection.getSectionForNode)(node);
    var replacementByteArray = (0, _wasmGen.encodeNode)(node);
    /**
     * Replace new node as bytes
     */

    uint8Buffer = (0, _helperBuffer.overrideBytesInBuffer)(uint8Buffer, // $FlowIgnore: assertNodeHasLoc ensures that
    node.loc.start.column, // $FlowIgnore: assertNodeHasLoc ensures that
    node.loc.end.column, replacementByteArray);
    /**
     * Update section size
     */

    var deltaBytes = replacementByteArray.length - ( // $FlowIgnore: assertNodeHasLoc ensures that
    node.loc.end.column - node.loc.start.column);
    uint8Buffer = (0, _helperWasmSection.resizeSectionByteSize)(ast, uint8Buffer, sectionName, deltaBytes); // Update new node end position
    // $FlowIgnore: assertNodeHasLoc ensures that

    node.loc.end.column = node.loc.start.column + replacementByteArray.length;
  });
  return uint8Buffer;
}

function applyToNodeToDelete(ast, uint8Buffer, nodes) {
  var deltaElements = -1; // since we removed an element

  nodes.forEach(function (node) {
    assertNodeHasLoc(node);
    var sectionName = (0, _helperWasmSection.getSectionForNode)(node); // replacement is nothing

    var replacement = [];
    uint8Buffer = (0, _helperBuffer.overrideBytesInBuffer)(uint8Buffer, // $FlowIgnore: assertNodeHasLoc ensures that
    node.loc.start.column, // $FlowIgnore: assertNodeHasLoc ensures that
    node.loc.end.column, replacement);
    /**
     * Update section
     */
    // $FlowIgnore: assertNodeHasLoc ensures that

    var deltaBytes = -(node.loc.end.column - node.loc.start.column);
    uint8Buffer = (0, _helperWasmSection.resizeSectionByteSize)(ast, uint8Buffer, sectionName, deltaBytes);
    uint8Buffer = (0, _helperWasmSection.resizeSectionVecSize)(ast, uint8Buffer, sectionName, deltaElements);
  });
  return uint8Buffer;
}

function applyToNodeToAdd(ast, uint8Buffer, nodes) {
  var deltaElements = +1; // since we added an element

  nodes.forEach(function (node) {
    var sectionName = (0, _helperWasmSection.getSectionForNode)(node);
    var sectionMetadata = (0, _ast.getSectionMetadata)(ast, sectionName); // Section doesn't exists, we create an empty one

    if (typeof sectionMetadata === "undefined") {
      var res = (0, _helperWasmSection.createEmptySection)(ast, uint8Buffer, sectionName);
      uint8Buffer = res.uint8Buffer;
      sectionMetadata = res.sectionMetadata;
    }
    /**
     * Add nodes
     */


    var newByteArray = (0, _wasmGen.encodeNode)(node); // start at the end of the section

    var start = sectionMetadata.startOffset + sectionMetadata.size + 1;
    var end = start;
    uint8Buffer = (0, _helperBuffer.overrideBytesInBuffer)(uint8Buffer, start, end, newByteArray);
    var deltaBytes = newByteArray.length;
    /**
     * Update section
     */

    uint8Buffer = (0, _helperWasmSection.resizeSectionByteSize)(ast, uint8Buffer, sectionName, deltaBytes);
    uint8Buffer = (0, _helperWasmSection.resizeSectionVecSize)(ast, uint8Buffer, sectionName, deltaElements);
  });
  return uint8Buffer;
}