"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.edit = edit;
exports.add = add;

var _wasmParser = require("@webassemblyjs/wasm-parser");

var _ast = require("@webassemblyjs/ast");

var _apply = require("./apply");

function hashPath(_ref) {
  var node = _ref.node;
  return JSON.stringify(node);
}

var decoderOpts = {
  ignoreCodeSection: true,
  ignoreDataSection: true
};

function edit(ab, visitors) {
  var nodesToDelete = [];
  var nodesToUpdate = [];

  if (typeof visitors.Instr === "function") {
    decoderOpts.ignoreCodeSection = false;
    console.warn("Decoding the code section has been enabled.");
  }

  var ast = (0, _wasmParser.decode)(ab, decoderOpts);
  var uint8Buffer = new Uint8Array(ab);
  var nodeBeforeHash = "";

  function before(type, path) {
    nodeBeforeHash = hashPath(path);
  }

  function after(type, path) {
    if (path.node._deleted === true) {
      nodesToDelete.push(path.node);
    } else if (nodeBeforeHash !== hashPath(path)) {
      nodesToUpdate.push(path.node);
    }
  }

  (0, _ast.traverseWithHooks)(ast, visitors, before, after);
  uint8Buffer = (0, _apply.applyToNodeToUpdate)(ast, uint8Buffer, nodesToUpdate);
  uint8Buffer = (0, _apply.applyToNodeToDelete)(ast, uint8Buffer, nodesToDelete);
  return uint8Buffer.buffer;
}

function add(ab, newNodes) {
  var ast = (0, _wasmParser.decode)(ab, decoderOpts);
  var uint8Buffer = new Uint8Array(ab);
  uint8Buffer = (0, _apply.applyToNodeToAdd)(ast, uint8Buffer, newNodes);
  return uint8Buffer.buffer;
}